==> paragraphs_summary_token-2.0.4.tar.gz: paragraphs_summary_token/src/Service/ImageBuilder.php <==
<?php

namespace Drupal\paragraphs_summary_token\Service;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\EntityReferenceFieldItemList;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
use Drupal\entity_reference_revisions\EntityReferenceRevisionsFieldItemList;
use Drupal\file\FileInterface;
use Drupal\file\Plugin\Field\FieldType\FileFieldItemList;
use Drupal\image\ImageStyleInterface;
use Drupal\media\MediaInterface;
use Drupal\paragraphs_summary_token\Traits\ParagraphsSummaryTokenTrait;

/**
 * The image builder.
 */
class ImageBuilder implements ImageBuilderInterface {

  use ParagraphsSummaryTokenTrait;

  /**
   * The file storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $fileStorage;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The stream wrapper manager.
   *
   * @var \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface
   */
  protected $streamWrapperManager;

  /**
   * SummaryBuilder constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The language manager.
   * @param \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface $streamWrapperManager
   *   The stream wrapper manager.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, LanguageManagerInterface $languageManager, StreamWrapperManagerInterface $streamWrapperManager) {
    $this->setFieldStorageConfigStorage($entityTypeManager->getStorage('field_storage_config'));
    $this->fileStorage = $entityTypeManager->getStorage('file');
    $this->languageManager = $languageManager;
    $this->streamWrapperManager = $streamWrapperManager;
  }

  /**
   * {@inheritdoc}
   */
  public function build(EntityReferenceRevisionsFieldItemList $paragraphs_field, ImageStyleInterface $image_style = NULL): string {
    $image_url = $this->retrieveImageUrlFromParagraphsField($paragraphs_field);

    if (empty($image_url)) {
      return '';
    }

    if ($image_style instanceof ImageStyleInterface) {
      return $image_style->buildUrl($image_url);
    }

    return file_create_url($image_url);
  }

  /**
   * Get the image URL from the given paragraphs field.
   *
   * @param \Drupal\entity_reference_revisions\EntityReferenceRevisionsFieldItemList $paragraphs_field
   *   The paragraphs field.
   *
   * @return string
   *   The image URL.
   */
  private function retrieveImageUrlFromParagraphsField(EntityReferenceRevisionsFieldItemList $paragraphs_field): string {
    $image_url = '';

    if (!$paragraphs_field->isEmpty()) {
      $language = $this->languageManager->getCurrentLanguage()->getId();
      /** @var \Drupal\paragraphs\ParagraphInterface $paragraph */
      foreach ($paragraphs_field->referencedEntities() as $paragraph) {
        // First, loop over all text_long fields and check if one of those
        // fields contain content.
        $fields = $this->getFieldsByEntityTypeAndFieldType($paragraph->getEntityType(), 'image');
        $fields = array_merge($fields, $this->getEntityReferenceFields($paragraph->getEntityType(), 'media'));
        foreach ($fields as $field) {
          if ($paragraph->hasField($field) &&
            !$paragraph->get($field)->isEmpty()
          ) {
            // Check if the paragraph has a translation.
            if ($paragraph->hasTranslation($language)) {
              $paragraph = $paragraph->getTranslation($language);
            }
            $image_url = $this->retrieveImageUrlFromField($paragraph->get($field));
            if (!empty($image_url)) {
              break 2;
            }
          }
        }

        // No summary found, check if the paragraph has a reference field to
        // add nested paragraphs.
        foreach ($this->getEntityReferenceFields($paragraph->getEntityType(), 'paragraph', 'entity_reference_revisions') as $paragraphs_field_name) {
          if ($paragraph->hasField($paragraphs_field_name)) {
            $image_url = $this->retrieveImageUrlFromParagraphsField($paragraph->get($paragraphs_field_name));
            if (!empty($image_url)) {
              break 2;
            }
          }
        }
      }
    }

    return $image_url;
  }

  /**
   * Retrieve the image URL from the given field.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface $field
   *   The field that contains an image.
   *
   * @return string
   *   The image URL.
   */
  private function retrieveImageUrlFromField(FieldItemListInterface $field): string {
    $image_uri = '';
    $file = NULL;
    if ($field instanceof FileFieldItemList && $field->entity instanceof FileInterface) {
      $file = $field->entity;
    }
    elseif ($field instanceof EntityReferenceFieldItemList && $field->entity instanceof MediaInterface) {
      $file = $this->retrieveFileFromMediaEntity($field->entity);
    }

    if ($file instanceof FileInterface) {
      $scheme = $this->streamWrapperManager->getScheme($file->getFileUri());

      if ($this->streamWrapperManager->isValidScheme($scheme) && file_exists($file->getFileUri())) {
        $image_uri = $file->getFileUri();
      }
    }

    return $image_uri;
  }

  /**
   * Retrieve the file entity from the given media entity.
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media entity.
   *
   * @return \Drupal\file\FileInterface|null
   *   The file when found, else NULL.
   */
  private function retrieveFileFromMediaEntity(MediaInterface $media): ?FileInterface {
    $source = $media->getSource();
    $config = $source->getConfiguration();
    $field = $config['source_field'];

    if (!$field) {
      return NULL;
    }

    $fid = $media->{$field}->target_id;

    if (!$fid) {
      return NULL;
    }

    $file = $this->fileStorage->load($fid);

    if (!$file instanceof FileInterface) {
      return NULL;
    }

    $errors = file_validate_is_image($file);

    if (!empty($errors)) {
      return NULL;
    }

    return $file;
  }

}

==> paragraphs_summary_token-2.0.5.tar.gz: paragraphs_summary_token/src/Service/ImageBuilder.php <==
<?php

namespace Drupal\paragraphs_summary_token\Service;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\EntityReferenceFieldItemListInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
use Drupal\entity_reference_revisions\EntityReferenceRevisionsFieldItemList;
use Drupal\file\FileInterface;
use Drupal\file\Plugin\Field\FieldType\FileFieldItemList;
use Drupal\image\ImageStyleInterface;
use Drupal\media\MediaInterface;
use Drupal\paragraphs_summary_token\Traits\ParagraphsSummaryTokenTrait;

/**
 * The image builder.
 */
class ImageBuilder implements ImageBuilderInterface {

  use ParagraphsSummaryTokenTrait;

  /**
   * The file storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $fileStorage;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The stream wrapper manager.
   *
   * @var \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface
   */
  protected $streamWrapperManager;

  /**
   * SummaryBuilder constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The language manager.
   * @param \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface $streamWrapperManager
   *   The stream wrapper manager.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, LanguageManagerInterface $languageManager, StreamWrapperManagerInterface $streamWrapperManager) {
    $this->setFieldStorageConfigStorage($entityTypeManager->getStorage('field_storage_config'));
    $this->fileStorage = $entityTypeManager->getStorage('file');
    $this->languageManager = $languageManager;
    $this->streamWrapperManager = $streamWrapperManager;
  }

  /**
   * {@inheritdoc}
   */
  public function build(EntityReferenceRevisionsFieldItemList $paragraphs_field, ImageStyleInterface $image_style = NULL): string {
    $image_url = $this->retrieveImageUrlFromParagraphsField($paragraphs_field);

    if (empty($image_url)) {
      return '';
    }

    if ($image_style instanceof ImageStyleInterface) {
      return $image_style->buildUrl($image_url);
    }

    return file_create_url($image_url);
  }

  /**
   * Get the image URL from the given paragraphs field.
   *
   * @param \Drupal\entity_reference_revisions\EntityReferenceRevisionsFieldItemList $paragraphs_field
   *   The paragraphs field.
   *
   * @return string
   *   The image URL.
   */
  private function retrieveImageUrlFromParagraphsField(EntityReferenceRevisionsFieldItemList $paragraphs_field): string {
    $image_url = '';

    if (!$paragraphs_field->isEmpty()) {
      $language = $this->languageManager->getCurrentLanguage()->getId();
      /** @var \Drupal\paragraphs\ParagraphInterface $paragraph */
      foreach ($paragraphs_field->referencedEntities() as $paragraph) {
        // First, loop over all text_long fields and check if one of those
        // fields contain content.
        $fields = $this->getFieldsByEntityTypeAndFieldType($paragraph->getEntityType(), 'image');
        $fields = array_merge($fields, $this->getEntityReferenceFields($paragraph->getEntityType(), 'media'));
        foreach ($fields as $field) {
          if ($paragraph->hasField($field) &&
            !$paragraph->get($field)->isEmpty()
          ) {
            // Check if the paragraph has a translation.
            if ($paragraph->hasTranslation($language)) {
              $paragraph = $paragraph->getTranslation($language);
            }
            $image_url = $this->retrieveImageUrlFromField($paragraph->get($field));
            if (!empty($image_url)) {
              break 2;
            }
          }
        }

        // No summary found, check if the paragraph has a reference field to
        // add nested paragraphs.
        foreach ($this->getEntityReferenceFields($paragraph->getEntityType(), 'paragraph', 'entity_reference_revisions') as $paragraphs_field_name) {
          if ($paragraph->hasField($paragraphs_field_name)) {
            $image_url = $this->retrieveImageUrlFromParagraphsField($paragraph->get($paragraphs_field_name));
            if (!empty($image_url)) {
              break 2;
            }
          }
        }
      }
    }

    return $image_url;
  }

  /**
   * Retrieve the image URL from the given field.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface $field
   *   The field that contains an image.
   *
   * @return string
   *   The image URL.
   */
  private function retrieveImageUrlFromField(FieldItemListInterface $field): string {
    $image_uri = '';
    $file = NULL;
    if ($field instanceof FileFieldItemList && $field->entity instanceof FileInterface) {
      $file = $field->entity;
    }
    elseif ($field instanceof EntityReferenceFieldItemListInterface && $field->entity instanceof MediaInterface) {
      $file = $this->retrieveFileFromMediaEntity($field->entity);
    }

    if ($file instanceof FileInterface) {
      $scheme = $this->streamWrapperManager->getScheme($file->getFileUri());

      if ($this->streamWrapperManager->isValidScheme($scheme) && file_exists($file->getFileUri())) {
        $image_uri = $file->getFileUri();
      }
    }

    return $image_uri;
  }

  /**
   * Retrieve the file entity from the given media entity.
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media entity.
   *
   * @return \Drupal\file\FileInterface|null
   *   The file when found, else NULL.
   */
  private function retrieveFileFromMediaEntity(MediaInterface $media): ?FileInterface {
    $source = $media->getSource();
    $config = $source->getConfiguration();
    $field = $config['source_field'];

    if (!$field) {
      return NULL;
    }

    $fid = $media->{$field}->target_id;

    if (!$fid) {
      return NULL;
    }

    $file = $this->fileStorage->load($fid);

    if (!$file instanceof FileInterface) {
      return NULL;
    }

    $errors = file_validate_is_image($file);

    if (!empty($errors)) {
      return NULL;
    }

    return $file;
  }

}
